package org.openxma.dsl.generator.helper;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.dom.model.ConditionsBlock;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.xmadsl.GuidesignExtension;

import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.flex.FlexFactory;
import at.spardat.xma.guidesign.flex.XMAApplicationContext;
import at.spardat.xma.guidesign.flex.XMAStateFlag;

public class StatusFlagExtension {
	
	/**
	 * Create a xma-statusflag from a xmadsl-statusflag
	 */	
	// TODO use in Extend
	public static XMAStateFlag createXmaStatusFlag(StatusFlag statusFlag, XMAComponent xmaComp, XMAPage currPage, DataObjectVariable dov) {

		XMAStateFlag xmaStatusFlag =FlexFactory.eINSTANCE.createXMAStateFlag();
		// TODO uncomment following lines
		//Expression xmaExpression = statusFlag.getExpression().createXMAExpression(xmaComp,currPage,dov);
		//String namXMAStateFlag = getXMAStateFlagName(statusFlag,dov);		
			
		//xmaStatusFlag.setNamState(namXMAStateFlag);
		//xmaStatusFlag.setCondition(xmaExpression);			

		return xmaStatusFlag;
	}
	
	/**
	 * Find statusflag in a XMAComponent.
	 */	
	// TODO use in Extend	
	public static XMAStateFlag findXMAStateFlagForComponent(XMAComponent xmaComponent, StatusFlag statusFlag, DataObjectVariable dov) {

		XMAStateFlag existingStateFlag = findXMAStateFlagInList(xmaComponent.getStateFlags(),statusFlag,dov);
		
		if (existingStateFlag == null) {
			return createAndAppendXMAStateFlagInList(xmaComponent.getStateFlags(),xmaComponent,null,dov,statusFlag);
		}
		return existingStateFlag;
	}
	
	/**
	 * Find statusflag in a XMAPage.
	 */	
	public static XMAStateFlag findXMAStateFlagForPage(XMAPage xmaPage, StatusFlag statusFlag, XMAComponent xmaComponent, DataObjectVariable dov) {
		XMAStateFlag existingStateFlag = findXMAStateFlagInList(xmaPage.getStateFlags(),statusFlag,dov);				
		if (existingStateFlag == null) {
			return createAndAppendXMAStateFlagInList(xmaPage.getStateFlags(),xmaComponent,xmaPage,dov,statusFlag);
		}
		return existingStateFlag;
	}
	
	/**
	 * Find statusflag in a Application.
	 */	
	public static XMAStateFlag findXMAStateFlagForApplication(XMAComponent xmaComponent, StatusFlag statusFlag, DataObjectVariable dov) {
		XMAApplicationContext application = GuidesignExtension.getXMAApplicationContext(xmaComponent);		
		XMAStateFlag existingStateFlag = findXMAStateFlagInList(application.getStateFlags(),statusFlag,dov);					
		if (existingStateFlag == null) {
			return createAndAppendXMAStateFlagInList(application.getStateFlags(),xmaComponent,null,dov,statusFlag);
		}
		return existingStateFlag;	
	}
	
	// TODO use in Extend	
	public static XMAStateFlag findXMAStateFlagInList(List<XMAStateFlag> xmaStateFlagList, StatusFlag statusFlag, DataObjectVariable dov) {
		String namXMAStateFlag = getXMAStateFlagName(statusFlag,dov);
		for (XMAStateFlag xmaStateFlag : xmaStateFlagList) {
			if (xmaStateFlag.getNamState().equals(namXMAStateFlag)) {
				return xmaStateFlag;
			}
		}
		return null;	
	}
	
	// TODO use in Extend	
	public static XMAStateFlag createAndAppendXMAStateFlagInList(List<XMAStateFlag> xmaStateFlagList, XMAComponent xmaComponent, XMAPage currPage, DataObjectVariable dov, StatusFlag statusFlag) {

		XMAStateFlag xmaStateFlag = createXmaStatusFlag(statusFlag,xmaComponent,currPage,dov);
		 
		xmaStateFlagList.add(xmaStateFlag);	
		return xmaStateFlag;	
	}
	
	// TODO use in Extend	
	public static String getXMAStateFlagName(StatusFlag statusFlag, DataObjectVariable dov) {
		EObject ownerOfStatusFlag = getOwnerOfStatusFlag(statusFlag);		
		if (ownerOfStatusFlag instanceof Entity) {  
			return "st_"+dov.getName()+"_"+statusFlag.getName();
		}		
		return "st_"+statusFlag.getName();
	}
	
	/**
	 * Get the owner of a StatusFlag (Component, Page or Entity)
	 */
	// TODO use in Extend	
	public static EObject getOwnerOfStatusFlag(StatusFlag statusFlag) {
		ConditionsBlock conditionsBlock = (ConditionsBlock)statusFlag.eContainer();	
		return conditionsBlock.eContainer();
	}	

}
